#!/usr/bin/env python3
"""
cli/domain.py Defines the "halfapi domain" cli commands
"""
# builtins
import sys

import click


from .cli import cli
from ..conf import config, write_config, DOMAINSDICT

from ..lib.schemas import schema_dict_dom
from ..lib.routes import api_routes


from ..logging import logger

#################
# domain create #
#################
def create_domain(domain_name: str, module_path: str):
    logger.info('Will add **%s** (%s) to current halfAPI project',
            domain_name, module_path)

    #if domain_name in DOMAINSDICT():
    #    logger.warning('Domain **%s** is already in project', domain_name)
    #    sys.exit(1)

    if not config.has_section('domains'):
        config.add_section('domains')

    config.set('domains', domain_name, module_path)
    write_config()


###############
# domain read #
###############
def list_routes(domain, m_dom):
    """
    Echoes the list of the **m_dom** active routes
    """

    click.echo(f'\nDomain : {domain}\n')

    routes = api_routes(m_dom)[0]
    if len(routes):
        for key, item in routes.items():
            methods = '|'.join(list(item.keys()))
            click.echo(f'\t{key} : {methods}')
    else:
        click.echo('\t**No ROUTES**')
        raise Exception('Routeless domain')



def list_api_routes():
    """
    Echoes the list of all active domains.
    """

    click.echo('# API Routes')
    for domain, m_dom in DOMAINSDICT().items():
        list_routes(domain, m_dom)


@click.option('--read',default=False, is_flag=True)
@click.option('--create',default=False, is_flag=True)
@click.option('--update',default=False, is_flag=True)
@click.option('--delete',default=False, is_flag=True)
@click.option('--domains',default=None)
@cli.command()
def domain(domains, delete, update, create, read):  #, domains, read, create, update, delete):
    """
    The "halfapi domain" command

    Parameters:
        domain (List[str]|None): The list of the domains to list/update

        The parameter has a misleading name as it is a multiple option
        but this would be strange to use it several times named as "domains"

        update (boolean): If set, update the database for the selected domains
    """

    dom_dict = DOMAINSDICT()

    if not domains:
        if create:
            # TODO: Connect to the create_domain function
            raise NotImplementedError
    else:
        dom_dict_ = {}

        for domain_name in domains.split(','):
            if domain_name in dom_dict.keys():
                dom_dict_[domain_name] = dom_dict[domain_name]
                continue

            click.echo(
                f'Domain {domain_name}s is not activated in the configuration')

        dom_dict = dom_dict_

    for domain_name, m_dom in dom_dict.items():
        if update:
            raise NotImplementedError
        if delete:
            raise NotImplementedError

        if read:
            list_routes(domain_name, m_dom)
